#include <CurieBLE.h>
#include <CurieTime.h>
#include "CurieTimerOne.h"
#include <EEPROM.h>
#define LED 13

BLEPeripheral blePeripheral;  // BLE Peripheral Device (the board you're programming)
BLEService TamirArduino("19B10010-E8F2-537E-4F6C-D104768A1214"); // BLE AnalogRead Service
// BLE LED Switch Characteristic - custom 128-bit UUID, read and writable by central
BLEUnsignedIntCharacteristic LEDStatus("19B10011-E8F2-537E-4F6C-D104768A1214", BLERead | BLEWrite  );
BLEUnsignedIntCharacteristic Smartphone("19B10012-E8F2-537E-4F6C-D104768A1214", BLERead | BLENotify  );
BLEUnsignedIntCharacteristic Current("19B10013-E8F2-537E-4F6C-D104768A1214", BLERead | BLEWrite  );
BLEUnsignedIntCharacteristic StartTime1("19B10014-E8F2-537E-4F6C-D104768A1214", BLERead | BLEWrite  );
BLEUnsignedIntCharacteristic EndTime1("19B10015-E8F2-537E-4F6C-D104768A1214", BLERead | BLEWrite  );

struct EepromData { // structure to save the data
  int StartH;   // Start hour
  int StartM;    // Start Minute
  int EndH;   // End hour
  int EndM;    // End Minute
  int CurrH;   // Current hour
  int CurrM;    // Current Minute
};
EepromData FromEEprom ; // structure to get the data  

int incom = 0;
int last_incom = 0;
int Flag = 1;
int FlagHour = 0;
int FlagMinute = 0;
int StartHour = 0;
int StartMinute = 0;
int EndHour = 0;
int EndMinute = 0;
int OnOff = 0;

long previousMillis = 0; // Store last time that checked if the time to output arrive
int Duration = 10000;    // duration of on time (ms)
int PulseWidth = 1000;   //  Pulse width in ms

int PulseOut = 3;  // giv the pulse for the solenoid
int PolChange = 2; // change polarity for the solenoid

void setup() {

 /* EEPROM.get(0,FromEEprom);
  StartHour=FromEEprom.StartH;
  StartMinute=FromEEprom.StartM;
  EndHour=FromEEprom.EndH;
  EndMinute=FromEEprom.EndM;
  setTime(FromEEprom.CurrH, FromEEprom.CurrM, 00, 11, 07, 2017);*/
  
  Serial.begin(9600);
  // set advertised local name and service UUID:
  blePeripheral.setLocalName("TamirArduino");
  blePeripheral.setAdvertisedServiceUuid(TamirArduino.uuid());
  // add service and characteristic:
  blePeripheral.addAttribute(TamirArduino);
  blePeripheral.addAttribute(LEDStatus);
  blePeripheral.addAttribute(Smartphone);
  blePeripheral.addAttribute(Current);
  blePeripheral.addAttribute(StartTime1);
  blePeripheral.addAttribute(EndTime1);
  blePeripheral.begin(); // begin advertising BLE Light service:

  Serial.print("Progrem Started :)");
  Serial.print(hour());
  Serial.println(minute());
  Serial.print(StartHour);
  Serial.println(StartMinute);
  Serial.print(EndHour);
  Serial.println( EndMinute );

  pinMode(LED, OUTPUT);      // set Light pin to output mode
  pinMode(PulseOut, OUTPUT);
  pinMode(PolChange, OUTPUT);
}

void loop() {
  BLECentral central = blePeripheral.central();  // listen for BLE peripherals to connect:
  if (central) { // if a central is connected to peripheral:
    Serial.print("Connected to central: ");
    Serial.println(central.address()); // print the central's MAC address:
    while (central.connected()) {  // while the central is still connected to peripheral:
      SolCon();
      if (Current.written()) {  // Recieve from the Smartphone the Current time
        Flag = Current.value();
        FlagHour = Flag / 100;
        FlagMinute = Flag % 100;
        setTime(FlagHour, FlagMinute, 00, 11, 07, 2017);
        Serial.print("Current Time:");
        Serial.print(hour());
        Serial.println(minute());       
      }
      else if ( StartTime1.written()) {   // Recieve from the Smartphone the start time
        Flag = StartTime1.value();
        StartHour = Flag / 100;
        StartMinute = Flag % 100;
        Serial.print("Start Time:");
        Serial.print(StartHour);
        Serial.println( StartMinute);
        //saveEEPROM();
      }
      else if ( EndTime1.written()) {    // Recieve from the Smartphone the End Time
        Flag = EndTime1.value();
        EndHour = Flag / 100;
        EndMinute = Flag % 100;
        Serial.print("End Time:");
        Serial.print(EndHour);
        Serial.println( EndMinute);
        saveEEPROM();
      }
      if (LEDStatus.written())  {
        incom = LEDStatus.value();
        Smartphone.setValue(incom);// data that sent to smartphone over BLE
        Serial.print("Turned on/off at : ");
         Serial.print(hour());
        Serial.println(minute());
        if (incom != last_incom) { // Manual operation od the solenoid
          if (incom == 49)  // Turn on the solenoid
            StateSol (HIGH);
          else             // Turn of the solenoid
            StateSol (LOW);
        }
        last_incom = incom;
      }
    }
  }
  SolCon();// Set output when time arrive
}

void SolCon() { // control the solenoid operation
  unsigned long currentMillis = millis();
  if (currentMillis - previousMillis > Duration) { // save the last time you blinked the LED
    previousMillis = currentMillis;
    if (hour() >= StartHour && minute() >= StartMinute && hour() <= EndHour && minute() <= EndMinute && StartHour !=0 &&StartMinute!=0) { // check if time arrive to turn on
      Serial.print("ON ON ON ");
      Serial.print("Current Time: ");
      Serial.print(hour());
      Serial.println(minute());
      if (OnOff == 0) { // 80ms pulse for the Solenoid
        StateSol (HIGH);
        OnOff = 1;
      }
    }
    else { // Turn off the Solenoid
      if (OnOff == 1) { // 80ms pulse for the Solenoid
        StateSol (LOW);
        OnOff = 0;
      }
    }
  }
}
void StateSol (boolean State) { // When State = HIGH its turn on the Solenoid and when State=LOW turn off with pulse
  digitalWrite(PolChange, State);//
  digitalWrite(LED, State);//
  digitalWrite(PulseOut, HIGH);
  delay (PulseWidth);
  digitalWrite(PulseOut, LOW);
}
void saveEEPROM(){
   Serial.println("First");
  EepromData ToEEprom = {
                 StartHour,
                 StartMinute,
                 EndHour,
                 EndMinute,
                 hour(),
                 minute()
                  };
                   Serial.println(ToEEprom.EndH);
                   Serial.println(ToEEprom.EndM);
                   Serial.println(ToEEprom.StartH);
                   Serial.println(ToEEprom.StartM);
        EEPROM.put(0, ToEEprom);
         Serial.println("Last");
}

